'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

require('es6-promise').polyfill();
require('isomorphic-fetch');

// REQUEST Methods
var METHODS = {
  GET: 'GET',
  POST: 'POST',
  PUT: 'PUT',
  DELETE: 'DELETE'
};

// Default Headers for JSON data
var defaultHeaders = {
  Accept: "application/json, text/plain, */*",
  "Content-Type": "application/json"
};

// CORS headers
var corsHeaders = {
  'Access-Control-Allow-Origin': '*'
};

/**
 * Converts an object to its url params representation.
 * @param {object} obj 
 * @returns {string} - the url encoded version of the object.
 */
var objectToParams = function objectToParams(obj) {
  return Object.keys(obj).map(function (k) {
    return k + '=' + obj[k];
  }).join("&");
};

var buildConfiguration = function buildConfiguration(config) {
  return Object.assign({}, DEFAULT_CONFIG, config);
};

/**
 * @typedef {Object} Config
 * @property {boolean} cors - to specify if it is a CORS request
 * @property {object} extraHeaders - extra headers to be sent in the request
 * @property {boolean} useDefaultHeaders - to specify if the default headers used
 */

var DEFAULT_CONFIG = {
  cors: false,
  extraHeaders: {},
  useDefaultHeaders: true
};

/**
 * Makes a Request sending JSON data
 * @param {string} url - endpoint url
 * @param {object} payload  - request's payload
 * @param {string} method - request method (GET, POST, PUT, DELETE)
 * @param {boolean} inBody - To specify if the payload goes in the body
 * @param {Config} configuration - configuration
 * @returns {Promise}
 */
var sendJSON = function sendJSON(url, payload) {
  var method = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : METHODS.GET;
  var inBody = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
  var configuration = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};

  var config = buildConfiguration(configuration);
  var fetchUrl = url;
  var options = {
    method: method,
    headers: _extends({}, config.useDefaultHeaders ? defaultHeaders : {}, config.extraHeaders, config.cors ? corsHeaders : {})
  };

  if (config.cors) {
    options.mode = 'cors';
  }

  if (inBody) {
    options.body = JSON.stringify(payload);
  } else {
    var params = objectToParams(payload);
    fetchUrl = url + '?' + params;
  }

  return fetch(fetchUrl, options).then(function (r) {
    return r.json();
  });
};

var requester = {
  /**
    *  Makes a GET request to the given url passing the payload data in the url
    * @param {string} url - endpoint url
    * @param {Object} payload  - request's payload
    * @param {Config} config - configuration object
    * @returns {Promise}
   */
  get: function get(url) {
    var payload = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var config = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    return sendJSON(url, payload, METHODS.GET, false, config);
  },
  /**
    *  Makes a POST request to the given url passing the payload data
    * @param {string} url - endpoint url
    * @param {Object} payload  - request's payload
    * @param {Config} config - configuration object
    * @returns {Promise}
   */
  post: function post(url) {
    var payload = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var config = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    return sendJSON(url, payload, METHODS.POST, true, config);
  },
  /**
    *  Makes a PUT request to the given url passing the payload data
    * @param {string} url - endpoint url
    * @param {Object} payload  - request's payload
    * @param {Config} config - configuration object
    * @returns {Promise}
   */
  put: function put(url) {
    var payload = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var config = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    return sendJSON(url, payload, METHODS.PUT, true, config);
  },
  /**
    *  Makes a DELETE request to the given url passing the payload data
    * @param {string} url - endpoint url
    * @param {Object} payload  - request's payload
    * @param {Config} config - configuration object
    * @returns {Promise}
   */
  delete: function _delete(url) {
    var payload = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var config = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    return sendJSON(url, payload, METHODS.DELETE, true, config);
  }
};

// Requester
module.exports = requester;